# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tab_create_params import TabCreateParams as TabCreateParams
from .tab_list_response import TabListResponse as TabListResponse
from .tab_create_response import TabCreateResponse as TabCreateResponse
from .tab_delete_response import TabDeleteResponse as TabDeleteResponse
from .tab_switch_response import TabSwitchResponse as TabSwitchResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ComputerCaptureScreenshotParams"]


class ComputerCaptureScreenshotParams(TypedDict, total=False):
    base64: bool

    tab_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .._models import BaseModel

__all__ = ["ComputerExecuteActionResponse", "PageContext"]


class PageContext(BaseModel):
    device_scale_factor: Optional[float] = None

    is_main_tab: Optional[bool] = None

    page_height: Optional[int] = None

    page_width: Optional[int] = None

    scroll_x: Optional[float] = None

    scroll_y: Optional[float] = None

    tab_id: Optional[str] = None

    title: Optional[str] = None

    url: Optional[str] = None

    viewport_height: Optional[int] = None

    viewport_width: Optional[int] = None


class ComputerExecuteActionResponse(BaseModel):
    error_message: Optional[str] = None

    executed_tab_id: Optional[str] = None

    page_context: Optional[PageContext] = None

    request_id: Optional[str] = None

    result: Optional[Dict[str, object]] = None

    status: Optional[str] = None

    timestamp: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .computers import (
    ComputersResource,
    AsyncComputersResource,
    ComputersResourceWithRawResponse,
    AsyncComputersResourceWithRawResponse,
    ComputersResourceWithStreamingResponse,
    AsyncComputersResourceWithStreamingResponse,
)

__all__ = [
    "ComputersResource",
    "AsyncComputersResource",
    "ComputersResourceWithRawResponse",
    "AsyncComputersResourceWithRawResponse",
    "ComputersResourceWithStreamingResponse",
    "AsyncComputersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .computer_drag_params import ComputerDragParams as ComputerDragParams
from .computer_click_params import ComputerClickParams as ComputerClickParams
from .computer_debug_params import ComputerDebugParams as ComputerDebugParams
from .computer_create_params import ComputerCreateParams as ComputerCreateParams
from .computer_drag_response import ComputerDragResponse as ComputerDragResponse
from .computer_key_up_params import ComputerKeyUpParams as ComputerKeyUpParams
from .computer_list_response import ComputerListResponse as ComputerListResponse
from .computer_click_response import ComputerClickResponse as ComputerClickResponse
from .computer_debug_response import ComputerDebugResponse as ComputerDebugResponse
from .computer_create_response import ComputerCreateResponse as ComputerCreateResponse
from .computer_get_html_params import ComputerGetHTMLParams as ComputerGetHTMLParams
from .computer_key_down_params import ComputerKeyDownParams as ComputerKeyDownParams
from .computer_key_up_response import ComputerKeyUpResponse as ComputerKeyUpResponse
from .computer_mouse_up_params import ComputerMouseUpParams as ComputerMouseUpParams
from .computer_navigate_params import ComputerNavigateParams as ComputerNavigateParams
from .computer_type_text_params import ComputerTypeTextParams as ComputerTypeTextParams
from .computer_get_html_response import ComputerGetHTMLResponse as ComputerGetHTMLResponse
from .computer_key_down_response import ComputerKeyDownResponse as ComputerKeyDownResponse
from .computer_mouse_down_params import ComputerMouseDownParams as ComputerMouseDownParams
from .computer_mouse_up_response import ComputerMouseUpResponse as ComputerMouseUpResponse
from .computer_navigate_response import ComputerNavigateResponse as ComputerNavigateResponse
from .computer_retrieve_response import ComputerRetrieveResponse as ComputerRetrieveResponse
from .computer_right_click_params import ComputerRightClickParams as ComputerRightClickParams
from .computer_type_text_response import ComputerTypeTextResponse as ComputerTypeTextResponse
from .computer_double_click_params import ComputerDoubleClickParams as ComputerDoubleClickParams
from .computer_keep_alive_response import ComputerKeepAliveResponse as ComputerKeepAliveResponse
from .computer_mouse_down_response import ComputerMouseDownResponse as ComputerMouseDownResponse
from .computer_press_hotkey_params import ComputerPressHotkeyParams as ComputerPressHotkeyParams
from .computer_set_viewport_params import ComputerSetViewportParams as ComputerSetViewportParams
from .computer_execute_batch_params import ComputerExecuteBatchParams as ComputerExecuteBatchParams
from .computer_right_click_response import ComputerRightClickResponse as ComputerRightClickResponse
from .computer_double_click_response import ComputerDoubleClickResponse as ComputerDoubleClickResponse
from .computer_execute_action_params import ComputerExecuteActionParams as ComputerExecuteActionParams
from .computer_press_hotkey_response import ComputerPressHotkeyResponse as ComputerPressHotkeyResponse
from .computer_set_viewport_response import ComputerSetViewportResponse as ComputerSetViewportResponse
from .computer_execute_batch_response import ComputerExecuteBatchResponse as ComputerExecuteBatchResponse
from .computer_scroll_viewport_params import ComputerScrollViewportParams as ComputerScrollViewportParams
from .computer_execute_action_response import ComputerExecuteActionResponse as ComputerExecuteActionResponse
from .computer_scroll_viewport_response import ComputerScrollViewportResponse as ComputerScrollViewportResponse
from .computer_capture_screenshot_params import ComputerCaptureScreenshotParams as ComputerCaptureScreenshotParams
from .computer_capture_screenshot_response import ComputerCaptureScreenshotResponse as ComputerCaptureScreenshotResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tzafon import Computer, AsyncComputer
from tests.utils import assert_matches_type
from tzafon.types.computers import (
    TabListResponse,
    TabCreateResponse,
    TabDeleteResponse,
    TabSwitchResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTabs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Computer) -> None:
        tab = client.computers.tabs.create(
            id="id",
        )
        assert_matches_type(TabCreateResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Computer) -> None:
        tab = client.computers.tabs.create(
            id="id",
            url="url",
        )
        assert_matches_type(TabCreateResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Computer) -> None:
        response = client.computers.tabs.with_raw_response.create(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tab = response.parse()
        assert_matches_type(TabCreateResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Computer) -> None:
        with client.computers.tabs.with_streaming_response.create(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tab = response.parse()
            assert_matches_type(TabCreateResponse, tab, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Computer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.computers.tabs.with_raw_response.create(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Computer) -> None:
        tab = client.computers.tabs.list(
            "id",
        )
        assert_matches_type(TabListResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Computer) -> None:
        response = client.computers.tabs.with_raw_response.list(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tab = response.parse()
        assert_matches_type(TabListResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Computer) -> None:
        with client.computers.tabs.with_streaming_response.list(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tab = response.parse()
            assert_matches_type(TabListResponse, tab, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Computer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.computers.tabs.with_raw_response.list(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Computer) -> None:
        tab = client.computers.tabs.delete(
            tab_id="tab_id",
            id="id",
        )
        assert_matches_type(TabDeleteResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Computer) -> None:
        response = client.computers.tabs.with_raw_response.delete(
            tab_id="tab_id",
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tab = response.parse()
        assert_matches_type(TabDeleteResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Computer) -> None:
        with client.computers.tabs.with_streaming_response.delete(
            tab_id="tab_id",
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tab = response.parse()
            assert_matches_type(TabDeleteResponse, tab, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Computer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.computers.tabs.with_raw_response.delete(
                tab_id="tab_id",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tab_id` but received ''"):
            client.computers.tabs.with_raw_response.delete(
                tab_id="",
                id="id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_switch(self, client: Computer) -> None:
        tab = client.computers.tabs.switch(
            tab_id="tab_id",
            id="id",
        )
        assert_matches_type(TabSwitchResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_switch(self, client: Computer) -> None:
        response = client.computers.tabs.with_raw_response.switch(
            tab_id="tab_id",
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tab = response.parse()
        assert_matches_type(TabSwitchResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_switch(self, client: Computer) -> None:
        with client.computers.tabs.with_streaming_response.switch(
            tab_id="tab_id",
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tab = response.parse()
            assert_matches_type(TabSwitchResponse, tab, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_switch(self, client: Computer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.computers.tabs.with_raw_response.switch(
                tab_id="tab_id",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tab_id` but received ''"):
            client.computers.tabs.with_raw_response.switch(
                tab_id="",
                id="id",
            )


class TestAsyncTabs:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncComputer) -> None:
        tab = await async_client.computers.tabs.create(
            id="id",
        )
        assert_matches_type(TabCreateResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncComputer) -> None:
        tab = await async_client.computers.tabs.create(
            id="id",
            url="url",
        )
        assert_matches_type(TabCreateResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncComputer) -> None:
        response = await async_client.computers.tabs.with_raw_response.create(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tab = await response.parse()
        assert_matches_type(TabCreateResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncComputer) -> None:
        async with async_client.computers.tabs.with_streaming_response.create(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tab = await response.parse()
            assert_matches_type(TabCreateResponse, tab, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncComputer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.computers.tabs.with_raw_response.create(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncComputer) -> None:
        tab = await async_client.computers.tabs.list(
            "id",
        )
        assert_matches_type(TabListResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncComputer) -> None:
        response = await async_client.computers.tabs.with_raw_response.list(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tab = await response.parse()
        assert_matches_type(TabListResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncComputer) -> None:
        async with async_client.computers.tabs.with_streaming_response.list(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tab = await response.parse()
            assert_matches_type(TabListResponse, tab, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncComputer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.computers.tabs.with_raw_response.list(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncComputer) -> None:
        tab = await async_client.computers.tabs.delete(
            tab_id="tab_id",
            id="id",
        )
        assert_matches_type(TabDeleteResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncComputer) -> None:
        response = await async_client.computers.tabs.with_raw_response.delete(
            tab_id="tab_id",
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tab = await response.parse()
        assert_matches_type(TabDeleteResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncComputer) -> None:
        async with async_client.computers.tabs.with_streaming_response.delete(
            tab_id="tab_id",
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tab = await response.parse()
            assert_matches_type(TabDeleteResponse, tab, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncComputer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.computers.tabs.with_raw_response.delete(
                tab_id="tab_id",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tab_id` but received ''"):
            await async_client.computers.tabs.with_raw_response.delete(
                tab_id="",
                id="id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_switch(self, async_client: AsyncComputer) -> None:
        tab = await async_client.computers.tabs.switch(
            tab_id="tab_id",
            id="id",
        )
        assert_matches_type(TabSwitchResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_switch(self, async_client: AsyncComputer) -> None:
        response = await async_client.computers.tabs.with_raw_response.switch(
            tab_id="tab_id",
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tab = await response.parse()
        assert_matches_type(TabSwitchResponse, tab, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_switch(self, async_client: AsyncComputer) -> None:
        async with async_client.computers.tabs.with_streaming_response.switch(
            tab_id="tab_id",
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tab = await response.parse()
            assert_matches_type(TabSwitchResponse, tab, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_switch(self, async_client: AsyncComputer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.computers.tabs.with_raw_response.switch(
                tab_id="tab_id",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tab_id` but received ''"):
            await async_client.computers.tabs.with_raw_response.switch(
                tab_id="",
                id="id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.computers import tab_create_params
from ...types.computers.tab_list_response import TabListResponse
from ...types.computers.tab_create_response import TabCreateResponse
from ...types.computers.tab_delete_response import TabDeleteResponse
from ...types.computers.tab_switch_response import TabSwitchResponse

__all__ = ["TabsResource", "AsyncTabsResource"]


class TabsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TabsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/tzafon/computer-python#accessing-raw-response-data-eg-headers
        """
        return TabsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TabsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/tzafon/computer-python#with_streaming_response
        """
        return TabsResourceWithStreamingResponse(self)

    def create(
        self,
        id: str,
        *,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TabCreateResponse:
        """
        Create a new tab, optionally navigating to a URL (browser sessions only)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/computers/{id}/tabs",
            body=maybe_transform({"url": url}, tab_create_params.TabCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TabCreateResponse,
        )

    def list(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TabListResponse:
        """
        Get a list of all open tabs with their IDs, URLs, titles, and main tab status
        (browser sessions only)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/computers/{id}/tabs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TabListResponse,
        )

    def delete(
        self,
        tab_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TabDeleteResponse:
        """Close a specific tab by ID.

        Cannot close the last remaining tab (browser
        sessions only)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not tab_id:
            raise ValueError(f"Expected a non-empty value for `tab_id` but received {tab_id!r}")
        return self._delete(
            f"/computers/{id}/tabs/{tab_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TabDeleteResponse,
        )

    def switch(
        self,
        tab_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TabSwitchResponse:
        """
        Switch the main/active tab to a different tab by ID (browser sessions only)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not tab_id:
            raise ValueError(f"Expected a non-empty value for `tab_id` but received {tab_id!r}")
        return self._post(
            f"/computers/{id}/tabs/{tab_id}/switch",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TabSwitchResponse,
        )


class AsyncTabsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTabsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/tzafon/computer-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTabsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTabsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/tzafon/computer-python#with_streaming_response
        """
        return AsyncTabsResourceWithStreamingResponse(self)

    async def create(
        self,
        id: str,
        *,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TabCreateResponse:
        """
        Create a new tab, optionally navigating to a URL (browser sessions only)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/computers/{id}/tabs",
            body=await async_maybe_transform({"url": url}, tab_create_params.TabCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TabCreateResponse,
        )

    async def list(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TabListResponse:
        """
        Get a list of all open tabs with their IDs, URLs, titles, and main tab status
        (browser sessions only)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/computers/{id}/tabs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TabListResponse,
        )

    async def delete(
        self,
        tab_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TabDeleteResponse:
        """Close a specific tab by ID.

        Cannot close the last remaining tab (browser
        sessions only)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not tab_id:
            raise ValueError(f"Expected a non-empty value for `tab_id` but received {tab_id!r}")
        return await self._delete(
            f"/computers/{id}/tabs/{tab_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TabDeleteResponse,
        )

    async def switch(
        self,
        tab_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TabSwitchResponse:
        """
        Switch the main/active tab to a different tab by ID (browser sessions only)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not tab_id:
            raise ValueError(f"Expected a non-empty value for `tab_id` but received {tab_id!r}")
        return await self._post(
            f"/computers/{id}/tabs/{tab_id}/switch",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TabSwitchResponse,
        )


class TabsResourceWithRawResponse:
    def __init__(self, tabs: TabsResource) -> None:
        self._tabs = tabs

        self.create = to_raw_response_wrapper(
            tabs.create,
        )
        self.list = to_raw_response_wrapper(
            tabs.list,
        )
        self.delete = to_raw_response_wrapper(
            tabs.delete,
        )
        self.switch = to_raw_response_wrapper(
            tabs.switch,
        )


class AsyncTabsResourceWithRawResponse:
    def __init__(self, tabs: AsyncTabsResource) -> None:
        self._tabs = tabs

        self.create = async_to_raw_response_wrapper(
            tabs.create,
        )
        self.list = async_to_raw_response_wrapper(
            tabs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tabs.delete,
        )
        self.switch = async_to_raw_response_wrapper(
            tabs.switch,
        )


class TabsResourceWithStreamingResponse:
    def __init__(self, tabs: TabsResource) -> None:
        self._tabs = tabs

        self.create = to_streamed_response_wrapper(
            tabs.create,
        )
        self.list = to_streamed_response_wrapper(
            tabs.list,
        )
        self.delete = to_streamed_response_wrapper(
            tabs.delete,
        )
        self.switch = to_streamed_response_wrapper(
            tabs.switch,
        )


class AsyncTabsResourceWithStreamingResponse:
    def __init__(self, tabs: AsyncTabsResource) -> None:
        self._tabs = tabs

        self.create = async_to_streamed_response_wrapper(
            tabs.create,
        )
        self.list = async_to_streamed_response_wrapper(
            tabs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tabs.delete,
        )
        self.switch = async_to_streamed_response_wrapper(
            tabs.switch,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import TypeAlias

from .._models import BaseModel

__all__ = ["ComputerListResponse", "ComputerListResponseItem"]


class ComputerListResponseItem(BaseModel):
    id: Optional[str] = None

    created_at: Optional[str] = None

    endpoints: Optional[Dict[str, str]] = None

    kind: Optional[str] = None

    status: Optional[str] = None


ComputerListResponse: TypeAlias = List[ComputerListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ComputerRightClickParams"]


class ComputerRightClickParams(TypedDict, total=False):
    tab_id: str

    x: float

    y: float



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ComputerDebugParams"]


class ComputerDebugParams(TypedDict, total=False):
    command: str

    max_output_length: int

    tab_id: str

    timeout_seconds: int
